package masterelection

import (
	"fmt"
	"math/rand"
	"testing"
	"time"

	"git.autistici.org/ale/autoradio/util"
)

func init() {
	rand.Seed(time.Now().Unix())
}

func countMasters(nodes []*MasterElection) int {
	var masters int
	for _, n := range nodes {
		if n.IsMaster() {
			masters++
		}
	}
	return masters
}

func verifyMasterData(t *testing.T, nodes []*MasterElection) {
	tmp := make(map[string]struct{})
	for _, n := range nodes {
		tmp[n.GetMasterData()] = struct{}{}
	}
	if len(tmp) != 1 {
		t.Errorf("master data propagation error: >1 values: %v", tmp)
	}
}

func TestMasterElection(t *testing.T) {
	//etcd := util.NewTestEtcdClient()
	etcd := util.NewTestEtcdClientWithLatency(20 * time.Millisecond)
	lockPath := "/master/election/test"

	n := 5
	var nodes []*MasterElection
	var stop []chan bool
	for i := 0; i < n; i++ {
		stopCh := make(chan bool)
		m := NewMasterElection(
			etcd,
			lockPath,
			fmt.Sprintf("%d", i),
			1,
			nil)
		m.LogPrefix = fmt.Sprintf("node%d: masterelection", i+1)
		go m.Run(stopCh)
		nodes = append(nodes, m)
		stop = append(stop, stopCh)
	}

	for i := 0; i < n; i++ {
		time.Sleep(100 * time.Millisecond)
		if nm := countMasters(nodes[i:len(nodes)]); nm != 1 {
			t.Errorf("@%d: masters=%d (expected <= 1)", i, nm)
		}
		verifyMasterData(t, nodes[i:len(nodes)])
		close(stop[i])
	}
}
